/**
 * @file 城市选择组件
 * @author  zhulei05
 * @date 2016-11-3
 * @date 2017-03-30
 */

define(['fusion/deps/fusion.best', 'fusion/deps/naboo', 'fusion/deps/spark', 'fusion/deps/cookie', 'fusion/deps/geolocation', 'fusion/deps/bdetpl'], function (register, Naboo, Spark, Cookie, geoLocation, etpl) {

    Naboo.register('moveX', function (next, dom, x, duration, ease) {
        var props = {
            'transform': 'translate3d(' + x + ', 0, 0)',
            '-webkit-transform': 'translate3d(' + x + ', 0, 0)'
        };
        Spark.css3(dom, props, duration, ease, 0, function () {
            next();
        });
    });

    /**
     * 点信息对象定义
     *
     * @date  2015-11-03
     *
     * @param {Object} option 配置参数
     * @param {number}   option.x         墨卡托坐标系X
     * @param {number}   option.y         墨卡托坐标系Y
     * @param {number}   option.accuracy  精度
     * @param {number}   option.cityCode  城市代码
     * @param {number}   option.timestamp 时间戳
     * @param {string}   option.name      当前位置名称
     * @param {string}   option.address   当前位置
     * @param {string}   option.addr      当前位置的 city district street信息
     */
    function Position(option) {
        option = option || {};

        this.x = parseFloat(option.x);
        this.y = parseFloat(option.y);
        this.accuracy = parseFloat(option.accuracy);
        this.cityCode = parseInt(option.cityCode, 10);
        this.timestamp = parseInt(option.timestamp, 10) || undefined;
        this.name = option.name || undefined;
        this.address = option.address || undefined;
        this.addr = option.addr || undefined;
    }

    /**
     * 获取cookie使用的字符串
     *
     * @return {string}
     */
    Position.prototype.getCookieString = function () {
        return this.x + '_' + this.y + '_' + this.accuracy + '_' + this.cityCode + '_' + this.timestamp;
    };

    Position.prototype.output = function () {
        return {
            x: this.x,
            y: this.y,
            accuracy: this.accuracy,
            cityCode: this.cityCode,
            timestamp: this.timestamp,
            name: this.name,
            address: this.address,
            addr: this.addr || {}
        };
    };

    Position.readFromCookie = function (cookie) {
        var info = cookie.split('_');
        // x, y, accuracy, cityCode, timestamp, name, address, addr
        return new Position({
            x: info[0],
            y: info[1],
            accuracy: info[2],
            cityCode: info[3],
            timestamp: info[4]
        });
    };

    Position.readFromObject = function (data) {
        return new Position(data);
    };

    /**
     * baiduloc cookie处理对象
     *
     * @type {Object}
     */
    var geoCookie = {
        domain: '.baidu.com',
        path: '/',
        key: 'BAIDULOC',

        /**
         * 过期时间
         * 48h, 2 * 24 * 3600 * 1000
         *
         * @type {number}
         */
        duration: 172800000,

        get: function () {
            if (!navigator.cookieEnabled) {
                return false;
            }

            var match = Cookie.cookie && Cookie.cookie().match(new RegExp(this.key + '=([^;]+);?'));

            if (match) {
                return Position.readFromCookie(match[1]);
            }

            return false;
        },

        set: function (position) {
            if (!navigator.cookieEnabled) {
                return false;
            }

            position = Position.readFromObject(position);
            position.timestamp = parseInt(Date.now(), 10);
            Cookie.cookie && Cookie.cookie(this.key, position.getCookieString(), {
                domain: this.domain,
                path: this.path,
                expires: this.duration
            });

            return true;
        },

        remove: function () {
            if (!navigator.cookieEnabled) {
                return false;
            }

            Cookie.cookie && Cookie.cookie(this.key, null);
        }
    };

    var BCity = register('b-city', {
        tpl: "<div class=\"b-city-wrapper\">\n    <div class=\"b-city-header\" <% if: (!!${headcolor})%>style=\"background-color:${headcolor}\"<% /if %>>\n        <div class=\"b-city-header-normal\">\n            ${title}\n            <i class=\"c-icon b-city-close\">&#xe61a;</i>\n            <div class=\"b-city-header-normal-search\">\n                <i class=\"c-icon c-gap-left-large c-color-gray\">&#xe737;</i>\n                <span>搜城市</span>\n            </div>\n        </div>\n        <div class=\"b-city-header-search\">\n            <input type=\"text\" placeholder=\"搜城市\" />\n            <i class=\"c-icon c-gray c-gap-left-large\">&#xe737;</i>\n            <button class=\"c-color-gray-a\">取消</button>\n        </div>\n    </div>\n    <div class=\"b-city-content\">\n        <div class=\"b-city-part b-city-part-loc\">\n            <div class=\"b-city-part-title\">定位</div>\n            <div class=\"c-row c-gap-top\">\n                <div class=\"c-span4\">\n                    <div class=\"b-city-btn b-city-btn-loc\" data-val=\"${location.code}\">\n                        ${location.name}\n                    </div>\n                </div>\n                <div class=\"c-span8\">\n                    <div class=\"b-city-reloc\">\n                        <i class=\"c-icon\">&#xe61d;</i>\n                        <span>重新定位</span>\n                    </div>\n                </div>\n            </div>\n        </div>\n        <div class=\"b-city-part b-city-part-history\">\n            <div class=\"b-city-part-title\">历史</div>\n            <div class=\"c-row c-gap-top\">\n                <div class=\"c-span4\">\n                </div>\n                <div class=\"c-span4\">\n                </div>\n                <div class=\"c-span4\">\n                </div>\n            </div>\n        </div>\n        <div class=\"b-city-part b-city-part-hot\">\n            <div class=\"b-city-part-title\">热门</div>\n        <% for: ${hot} as ${line} %>\n            <div class=\"c-row c-gap-top\">\n            <% for: ${line} as ${city} %>\n                <div class=\"c-span4\">\n                <% if: ${city} %>\n                    <div class=\"b-city-btn\" data-val=\"${city.code}\">${city.name}</div>\n                <% /if %>\n                </div>\n            <% /for %>\n            </div>\n        <% /for %>\n        </div>\n        <% for: ${list} as ${l}, ${key} %>\n        <div class=\"b-city-part b-city-part-letter b-city-part-${key} c-gap-top-large\">\n            <div class=\"b-city-part-title c-gap-bottom-small\">${key}</div>\n            <% for: ${l} as ${city} %>\n            <div class=\"b-city-part-list-item c-line-bottom\" data-val=\"${city.code}\">${city.name}</div>\n            <% /for %>\n        </div>\n        <% /for %>\n    </div>\n    <div class=\"b-city-sidebar\">\n        <div class=\"b-city-link\" data-anchor=\"loc\">定位</div>\n        <div class=\"b-city-link\" data-anchor=\"hot\">热门</div>\n        <% for: ${list} as ${l}, ${key} %>\n        <div class=\"b-city-link\" data-anchor=\"${key}\">${key}</div>\n        <% /for %>\n    </div>\n    <div class=\"b-city-letter-top\"></div>\n    <div class=\"b-city-large-char\">A</div>\n    <div class=\"b-city-sug b-city-mask\">\n    </div>\n</div>\n",
        css: ".b-city-wrapper {  position: fixed;  left: 0;  top: 0;  display: none;  width: 100%;  height: 100%;  z-index: 900;  background: #fff;  text-align: left;}.b-city-header {  position: absolute;  left: 0;  top: 0;  -webkit-box-sizing: border-box;  box-sizing: border-box;  width: 100%;  height: 44px;  background: #0099ff;  padding: 0 16px;  z-index: 100;}.b-city-header-normal {  position: relative;  height: 44px;  line-height: 44px;  color: #fff;  font-size: 20px;  text-align: center;}.b-city-close {  position: absolute;  left: 0;  top: 0;}.b-city-header-normal-search {  position: absolute;  top: 10px;  right: 0;  width: 100px;  height: 25px;  background: #fff;  border-raduis: 13px;  -webkit-border-radius: 13px;  text-align: left;  font-size: 14px;  line-height: 25px;}.b-city-header-normal-search span {  font-size: 13px;  color: #999;}.b-city-header-search {  display: none;  position: relative;  height: 25px;  line-height: 25px;  background: #fff;  border-radius: 13px;  margin-top: 10px;}.b-city-header-search input {  width: 100%;  height: 25px;  padding: 6px 0 6px 29px;  outline: none;  font-size: 13px;}.b-city-header-search i {  position: absolute;  left: 0;  top: 2px;}.b-city-header-search button {  position: absolute;  right: 0;  top: 0;  height: 25px;  padding: 0 14px;  font-size: 13px;}.b-city-header-search button:before {  content: '';  position: absolute;  left: 0;  top: 6px;  width: 0;  height: 13px;  border-left: 1px solid #ddd;}.b-city-sug {  display: none;  position: absolute;  left: 0;  top: 0;  width: 100%;  height: 100%;  padding: 44px 16px 0 16px;  -webkit-box-sizing: border-box;  box-sizing: border-box;  z-index: 50;  overflow: scroll;  background: #fff;}.b-city-sug.b-city-mask {  background: rgba(0, 0, 0, 0.3);}.b-city-search-item {  height: 42px;  line-height: 42px;}.b-city-content {  padding: 44px 16px 30px 16px;  width: 100%;  height: 100%;  color: #333;  -webkit-box-sizing: border-box;  box-sizing: border-box;  overflow-x: hidden;  overflow-y: scroll;  -webkit-overflow-scrolling: touch;}.b-city-part {  margin-top: 18px;}.b-city-part-loc,.b-city-part-history,.b-city-part-hot {  padding-right: 29px;}.b-city-part-history {  display: none;}.b-city-btn {  height: 38px;  line-height: 38px;  background: #f8f8f8;  text-align: center;}.b-city-reloc {  display: inline-block;  height: 38px;  line-height: 38px;}@-webkit-keyframes b-city-relocing-rotation {  from {    -webkit-transform: rotate(1deg);  }  to {    -webkit-transform: rotate(360deg);  }}.b-city-reloc i {  display: inline-block;}.b-city-relocing i {  -webkit-animation: b-city-relocing-rotation 1s ease 0s infinite normal;}.b-city-part-list-item {  height: 39px;  line-height: 39px;}.b-city-sidebar {  position: absolute;  right: 7px;  top: 46px;  width: 38px;  color: #666;  text-align: center;  z-index: 50;  -webkit-tap-highlight-color: transparent;}.b-city-link {  font-size: 10px;  line-height: 18px;}.b-city-letter-top {  display: none;  position: absolute;  top: 44px;  left: 0;  width: 100%;  height: 50px;  padding-left: 16px;  line-height: 50px;  background: #fff;  z-index: 40;  -webkit-box-sizing: border-box;  box-sizing: border-box;}.b-city-large-char {  display: none;  position: absolute;  left: 50%;  top: 50%;  width: 78px;  height: 78px;  line-height: 78px;  margin-left: -39px;  margin-top: -39px;  border-radius: 3px;  font-size: 36px;  color: #fff;  text-align: center;  background: rgba(51, 51, 51, 0.4);}",
        props: {
            headcolor:{
                type:String,
                value:'',
                onChange:function(){
                    this.render();
                }
            },
            title: {
                type: String,
                value: '选择城市'
            },
            hot: {
                type: Array,
                value: [],
                onChange: function () {
                    this.render();
                }
            },
            list: {
                type: Array,
                value: [],
                onChange: function () {
                    this.render();
                }
            },
            location: {
                type: Object,
                value: {},
                onChange: function () {
                    this.render();
                }
            }
        },
        // 包含所有城市信息的一个数组
        all: [],
        // 定位功能中显示的城市规则
        locationShowType: 'city',
        // 数据预处理
        preprocess: function () {
            var data = this.getProp();
            var hot = data.hot;
            var lineSize = 3;
            var num = Math.ceil(hot.length / lineSize);
            var arr = Array(lineSize * num - hot.length);
            hot = hot.concat(arr);
            var hot_seperate = [];
            for (var i = 0; i < num; i++) {
                hot_seperate[i] = hot.slice(i * lineSize, (i + 1) * lineSize);
            }
            // 把data.list扁平化，去掉key
            // 每个key对应的数组合并到同一个数组里
            var all = [];
            Object.keys(data.list).forEach(function (e, i) {
                all = all.concat(data.list[e]);
            });
            this.all = all;
            if (data.location && data.location.showType) {
                this.locationShowType = data.location.showType;
            }

            return {
                headcolor: data.headcolor,
                title: data.title,
                hot: hot_seperate,
                list: data.list,
                location: data.location
            };
        },
        // 重写render方法，实现数据渲染前的预处理
        render: function () {
            if (!this.constructor.styled && this.css) {
                var styleTag = document.createElement('style');
                styleTag.setAttribute('data-for', 'fusion');
                styleTag.innerText = this.css;
                document.querySelector('head').appendChild(styleTag);
                this.constructor.styled = true;
            }

            if (this.holdChild) {
                return;
            }

            // render html
            var render = etpl.compile(this.tpl);
            var data = this.preprocess();
            this.element.innerHTML = render(data);

            if (data.location.name && data.location.code) {
                return;
            }

            var _this = this;
            var pos = geoCookie.get();

            // 如果cookie中存在定位信息
            if (pos) {
                geoLocation.mercatorToAddress({
                    x: pos.x,
                    y: pos.y,
                    onSuccess: function (data) {
                        if (data && data.addr && data.addr.city && data.addr.city_code) {
                            var showName = pos.addr.city;
                            var showKey = showName;
                            if (_this.locationShowType === 'district' && pos.addr.district) {
                                showName = pos.addr.district;
                                /*区县会有重名，如果参数设置只显示区域则增加showKey值为城市+区域组合，区分重名*/
                                showKey += showName;
                            }
                            _this._setLocBtn({
                                name: showName,
                                code: pos.addr.city_code,
                                key: showKey
                            });
                        }
                        else {
                            _this._setLocBtn(_this.defaultLoc);
                        }
                    },
                    onFail: function (data) {
                        _this._setLocBtn(_this.defaultLoc);
                    }
                });
            }
            else {
                this._setLocBtn(_this.defaultLoc);
            }
        },
        defaultLoc: {
            name: '北京',
            code: 131
        },
        // 设置定位按钮的样式
        _setLocBtn: function (info) {
            var $locBtn = this.$('.b-city-btn-loc');
            $locBtn.text(info.name);
            $locBtn.data('val', info.code);
            if (info.key) {
                $locBtn.data('key', info.key);
            }
            if (info.disable) {
                $locBtn.addClass('c-color-gray');
            }
            else {
                $locBtn.removeClass('c-color-gray');
            }
        },
        init: function () {
            var _this = this;
            setTimeout(function () {
                _this.bindEvents();
            });
        },
        bindEvents: function () {
            var _this = this;

            this.$('.b-city-close').on('click', function () {
                _this.close();
            });

            this.$('.b-city-header-normal-search').on('click', function () {
                _this.$('.b-city-header-normal').hide();
                _this.$('.b-city-header-search').show();
                _this.$('.b-city-sug').show();
                _this.$('.b-city-header-search input').focus();
            });

            this.$('.b-city-header-search button, .b-city-mask').on('click', function () {
                _this.$('.b-city-header-normal').show();
                _this.$('.b-city-header-search').hide();
                _this.$('.b-city-sug').hide().addClass('b-city-mask').html('');
                _this.$('.b-city-header-search input').val('');
            });

            this.$('.b-city-content').on('scroll', function () {
                var headerHei = _this.$('.b-city-header').height();
                var $letterTop = _this.$('.b-city-letter-top');
                var last = false;
                var $titles = _this.$('.b-city-part-letter .b-city-part-title');
                for (var i = 0, len = $titles.length; i < len; i++) {
                    var $title = $titles.eq(i);
                    var top = $title.position().top;
                    // 字母置顶浮层行高50px，字体大小14px
                    // 所以文字上下还有18px空间
                    if (top <= (headerHei + 18)) {
                        last = $title.text();
                    }
                    else {
                        var letterTopHei = $letterTop.height();
                        if (top < (letterTopHei + headerHei)) {
                            $letterTop.css('top', top - letterTopHei);
                        }
                        else {
                            $letterTop.css('top', headerHei);
                        }
                        break;
                    }
                }
                if (last) {
                    $letterTop.text(last).show();
                }
                else {
                    $letterTop.hide();
                }
            });

            this.$('.b-city-sidebar').on('touchmove', function (e) {
                var y = e.touches[0].clientY;
                y -= parseInt($(this).css('top'));
                var $links = $(this).find('.b-city-link');
                var perHei = $links.eq(0).height();
                var index = Math.floor(y / perHei);
                var anchor = $links.eq(index).data('anchor');
                var char = $links.eq(index).text();
                if (anchor) {
                    var headerHei = _this.$('.b-city-header').height();
                    var $wrapper = _this.$('.b-city-content');
                    var top = _this.$('.b-city-part-' + anchor).position().top;
                    $wrapper.scrollTop($wrapper.scrollTop() + top - headerHei);
                    _this._showLargeChar(char);
                }
                return false;
            });

            this.$('.b-city-sidebar').on('click', '.b-city-link', function () {
                var $link = $(this);
                var anchor = $link.data('anchor');
                var char = $link.text();
                if (anchor) {
                    var headerHei = _this.$('.b-city-header').height();
                    var $wrapper = _this.$('.b-city-content');
                    var top = _this.$('.b-city-part-' + anchor).position().top;
                    $wrapper.scrollTop($wrapper.scrollTop() + top - headerHei);
                    _this._showLargeChar(char);
                }
                return false;
            });

            this.$('.b-city-header-search input').on('input', function () {
                var $sug = _this.$('.b-city-sug');
                var me = this;
                _this.timer && clearTimeout(_this.timer);
                // 函数节流
                _this.timer = setTimeout(function () {
                    var query = $(me).val();
                    if (query == '') {
                        $sug.hasClass('b-city-mask') ? '' : $sug.addClass('b-city-mask');
                        $sug.html('');
                        return;
                    }
                    var list = _this.all.filter(function (e, i) {
                        if (e.name.indexOf(query) > -1 || e.pinyin.toLowerCase().indexOf(query.toLowerCase()) > -1) {
                            return true;
                        }
                        return false;
                    });
                    $sug.hasClass('b-city-mask') ? $sug.removeClass('b-city-mask') : '';
                    _this.renderSearchList(list, query);
                }, 300);
            }).on('focus', function () {
                _this.$('.b-city-wrapper').css('position', 'absolute');
            }).on('blur', function () {
                _this.$('.b-city-wrapper').css('position', 'fixed');
            });

            this.$('.b-city-wrapper').on('click', '.b-city-btn, .b-city-part-list-item, .b-city-search-item', function () {
                var code = $(this).data('val');
                if (code == '') {
                    return;
                }
                _this.close();
                var name = $(this).text();
                var info = {
                    name: name,
                    code: code
                };
                var cityKey = $(this).data('key');
                if (cityKey) {
                    info.key = cityKey;

                }
                // history去重
                var arr = [info];
                _this.history.forEach(function (obj, i) {
                    if (obj.code != info.code) {
                        arr.push(obj);
                    }
                });
                _this.history = arr;
                _this.history.slice(0, 3);
                _this.history.forEach(function (obj, i) {
                    obj && _this.$('.b-city-part-history .c-span4').eq(i).html('<div class="b-city-btn" data-val="' + obj.code + '"">' + obj.name + '</div>');
                });
                _this.$('.b-city-part-history').show();
                var $links = _this.$('.b-city-link');
                if ($links.eq(1).data('anchor') != 'history') {
                    $links.eq(0).after('<div class="b-city-link" data-anchor="history">历史</div>');
                }
                _this.trigger('selected', info);
            }).on('touchmove', '.b-city-sug', function (e) {
                _this.$('.b-city-header-search input').blur();
            });

            this.$('.b-city-reloc').on('click', function () {
                if (geoLocation && geoLocation.getPosition) {
                    $(this).addClass('b-city-relocing');
                    $(this).find('span').text('定位中...');
                    var me = this;
                    geoLocation.getPosition({
                        onSuccess: function (pos) {
                            if (pos.addr && pos.addr.city && pos.addr.city_code) {
                                var showName = pos.addr.city;
                                var showKey = showName;
                                if (_this.locationShowType === 'district' && pos.addr.district) {
                                    showName = pos.addr.district;
                                    /*区县会有重名，如果参数设置只显示区域则增加showKey值为城市+区域组合，区分重名*/
                                    showKey += showName;
                                }
                                _this._setLocBtn({
                                    name: showName,
                                    code: pos.addr.city_code,
                                    key: showKey
                                });
                            }
                            else {
                                _this._setLocBtn({
                                    name: '定位失败',
                                    code: '',
                                    disable: true
                                });
                            }
                            $(me).removeClass('b-city-relocing');
                            $(me).find('span').text('重新定位');
                        },
                        onFail: function () {
                            $(me).removeClass('b-city-relocing');
                            $(me).find('span').text('重新定位');
                            _this._setLocBtn({
                                name: '定位失败',
                                code: '',
                                disable: true
                            });
                        }
                    });
                }
            });
        },
        renderSearchList: function (list, query) {
            var str = '';
            list.forEach(function (e, j) {
                str += '<div class="b-city-search-item';
                if (j + 1 < list.length) {
                    str += ' c-line-bottom';
                }
                str += '" data-val=' + e.code + '>';
                var name = e.name.replace(query, '<span class="c-color-gray">' + query + '</span>');
                str += name;
                str += '</div>';
            });
            this.$('.b-city-sug').html(str);
        },
        timer: null,
        // 当前回话的搜索历史
        // 关闭页面后即失效
        // 类似于sessionStorage
        history: [],
        _showLargeChar: function (char) {
            this.timer && clearTimeout(this.timer);
            var $largeChar = this.$('.b-city-large-char');
            $largeChar.text(char).show();
            this.timer = setTimeout(function () {
                $largeChar.hide();
            }, 1000);
        },
        open: function () {
            var $wrapper = this.$('.b-city-wrapper');
            $wrapper.css({
                'transform': 'translate3d(100%, 0, 0)',
                '-webkit-transform': 'translate3d(100%, 0, 0)'
            }).show();
            Naboo.moveX($wrapper.get(0), 0, 200, 'ease').start(function () {
                $wrapper.css({
                    'transform': 'none',
                    '-webkit-transform': 'none'
                });
            });
        },
        close: function () {
            this.$('.b-city-wrapper').hide();
            this.trigger('closed');
        },
        detached: function () {
            Spark.dispose();
        }
    });

    return BCity;
});