/**
 * h5 video播放器
 *
 * @file bdPlayer.js
 * @author songgenlei@baidu.com
 */
define('pmd/player/videoPlayer', ['require', 'pmd/deps/aio'], function(require) {

    // 加载videoplayer公共css
    $('<style data-for="pmd/player/videoPlayer"></style>').text("/**\n * @file 播放器组件样式\n * @author zhaolei09\n * @description 通过fis的__inline编译到js中\n */\n\n/*video 标签*/\n.c-videoplayer {\n    width: 100%;\n}\n\n/*ios为全屏播放，当前ui中不显示视频*/\n/*设置父级节点为position: relative || absolute，视频定位在居中位置，可从容器中间展开全屏播放*/\n.c-videoplayer-box-ios {\n    position: absolute;\n    top: 50%;\n    left: 50%;\n    width: 1px;\n    height: 1px;\n    overflow: hidden;\n    z-index: -1;\n}\n\n.c-videoplayer-box-ios video {\n    width: 1px;\n    height: 1px;\n}\n\n\n/*android 全屏弹窗容器*/\n.c-popup-wrapper .c-videoplayer-popup {\n    background: #000;\n    color: #fff;\n}\n\n.c-videoplayer-popup video {\n    width: 100%;\n    max-height: 70%;\n    position: absolute;\n    top: 50%;\n    left: 50%;\n    margin-left: -50%;\n    visibility: hidden;\n    /*display: none;*/\n}\n\n\n.c-videoplayer-popup .c-popup-content {\n    height: 100%;\n}\n\n.c-videoplayer-popup .c-loading {\n    position: absolute;\n    top: 0;\n    left: 0;\n    right: 0;\n    bottom: 0;\n    margin: auto;\n    height: 54px;\n}\n\n.c-videoplayer-popup .c-loading i::before {\n    border-top-color: #aaa;\n}\n").appendTo('head');


    /**
     * 构造函数
     *
     * @param {Object}              opt 配置对象
     * @param {Object}              opt.ios   可选 单独配置ios下某个表现形式
     * @param {boolean}             opt.ios.playMode  可选 播放模式
     *** 播放模式字段值说明：
     *** hide 容器中对video的显示进行隐藏 （ios默认为全屏播放，针对ui设定 当前播放区域不显示video）
     *** false || undefined 不使用插件带有的播放容器样式
     * @param {Object}              opt.android   可选  单独配置android下某个表现形式
     * @param {boolean}             opt.android.playMode  可选 播放模式
     *** 播放模式字段值说明：
     *** fullsreen 全屏 (android模拟全屏播放)
     *** false || undefined 不使用插件带有的播放容器样式
     */
    function VideoPlayer(opt) {
        var opt = this.options = $.extend(true, {
            height: null,
            ios: {
                playMode: false,
                playInBox: false
            },
            android: {
                playMode: false
            }
        }, opt);
        this.init(opt);
    }

    VideoPlayer.prototype = {
        constructor: VideoPlayer,
        init: function(opt) {
            var self = this;
            var ua = navigator.userAgent;
            self.video = '<video class="c-videoplayer" src="" type="video/mp4"'
                        + ' autoplay="autoplay" controls="" poster="" preload="none"></video>';
            self.$video = $(self.video);
            self.video = self.$video[0];
            // 播放器是否在工作，pause也算工作状态
            self.isWorking = false;
            
            if (opt.height) {
                self.video.height = opt.height;
            }

            self.bindEvents();
        },
        bindEvents: function() {
            var self = this;
            self.$video.on('loadeddata', function() {
                self.video.play();
            });
            self.$video.on('timeupdate', function() {
                for (var s in self.timeEvents) {
                    if (self.timeEvents.hasOwnProperty(s)) {
                        var th = self.timeEvents[s];
                        var currentTime = Math.ceil(self.video.currentTime);
                        var duration = self.getVideoTime();
                        if (th && duration && (currentTime / duration >= th)) {
                            self.trigger(s);
                            self.timeEvents[s] = 0;
                        }
                    }
                }
            });
        },
        // 关键播放时间点设置
        setTimeEvents: function() {
            this.timeEvents = {
                dataready: 0.0000001,
                onefifth: 0.2,
                fourfifths: 0.8,
                onehalf: 0.5,
                playend: 1
            };
        },

        // 初始化一些参数设置
        initParams: function () {
            var self = this;
            self.dataready = 0;
            self.playedInBox = false;
        },

        play: function(opt) {
            var self = this;

            self.initParams();

            // 插入视频节点
            self.appendVideo(opt);

            self.setTimeEvents();

            // 控制播放器播放
            // 手百 ios 8.0 以上调用客户端播放器播放，不需再次触发play
            if (!self.playedInBox) {
                self.playSingle();
            }

            self.isWorking = true;
        },

        // 控制播放器播放
        playSingle: function () {
            var self = this;
            var uaReg = /(iphone.+mqqbrowser)|(android.*(baidubrowser)|(baiduboxapp))/i;
            if (navigator.userAgent.match(uaReg)) {
                setTimeout(function() {
                    self.video.play();
                }, 30);
            } else {
                self.video.play();
            }
        },

        // 设置视频相关信息
        setVideo: function (options) {
            var video = this.$video;

            if (options.height || options.height === 0) {
                this.video.height = options.height;
            } else {
                this.video.removeAttribute('height');
            }

            // 设置视频资源地址
            video.attr('src', options.src || '');

            // 设置视频封面
            video.attr('poster', options.poster || '');
        },

        // 插入视频节点，根据playmode参数进行不同的设置
        appendVideo: function (opt) {
            var self = this;
            var video = self.$video;

            // 设置参数
            var options = $.extend({}, self.options, opt);

            // 使用手百NA播放器，此处代码需要放置在最前面，否则会导致被系统自带播放器覆盖的情况
            // 获取UA
            require('pmd/deps/aio');
            // 手百 ios 8.0以上版本 则使用手百自带播放器
            if (options.ios && options.ios.playInBox && window.Box && window.Box.isBox &&  window.Box.utils 
                && $.os && $.os.ios) {
                var curver = Box.utils.getVersion();
                var minver = '8.0.0.6';
                var versionCompare = Box.utils.version_compare;
                if (versionCompare(curver, minver) >= 0) {
                    try {
                        self.playedInBox = true;
                        self.playInBox(options);
                        return;
                    } catch(e) {
                        self.playedInBox = false;
                    }
                }
            }

            var container = $(options.container);

            // 设置视频相关信息
            self.setVideo(options);

            // 如果设置了playMode 且 ios 进行如下操作
            // ios不区分fullscreen || inline, 均为为默认系统全屏播放
            if ($.os && $.os.ios && options.ios && options.ios.playMode === 'hide') {
                var html = $('<div class="c-videoplayer-box-ios"></div>').append(video);
                container.length && container.append(html);
                return;
            }

            // 非ios下
            // 全屏播放
            if ((!$.os || !$.os.ios) && options.android
                // 容错 首次开发少写了一个c
                && (options.android.playMode === 'fullsreen' || options.android.playMode === 'fullscreen')) {
                video.on('dataready', function () {
                    self.dataready = 1;
                });
                self.fullsreenForAndriod(opt);
                return;
            }

            // 非ios下
            // 当前区域播放
            // 暂无当前播放规范 尚未开发
            // if (options.playMode === 'inline') {

            // }

            // 如果未设置playMode 则只会简单的插入video节点
            container.length && container.append(video);
        },

        fullsreenForAndriod: function (opt) {
            var self = this;
            var video = self.$video;

            // 设置参数
            var options = $.extend({}, self.options, opt);

            // QQ browser由于自有实现的播放器问题 特殊处理
            // 避免2次点击关闭按钮
            // if (navigator.userAgent.match(/(android.+mqqbrowser)/i)) {
            //     return;
            // }
            // 删除上面的处理 最新的6.8版本播放问题

            // 引用通用弹窗组件
            require(['pmd/popup/popup'], function (Popup) {
                var popup = new Popup({
                    title: options.title || '',
                    content: '<div class="c-loading">\
                                <i class="c-icon">&#xe780</i>\
                                <p>加载中...</p>\
                            </div>',
                    fullView: true,
                    customClassName: 'c-videoplayer-popup',
                    onOpen: function () {
                        // webkit核下 android浏览器 加载较慢 会展示控制条体验问题 增加loading优化
                        if (self.dataready === 1) {
                            addVideo();
                        } else {
                            video.on('dataready', function () {
                                addVideo();
                                self.dataready = 1;
                            });
                        }

                        // 如果超过某个时间（6s） 还未展示 则立即展示出来
                        clearLoadingTimer();
                        self.loadingTimer = setTimeout(function () {
                            if (self.dataready !== 1) {
                                addVideo();
                                self.dataready = 1;
                            }
                        }, 6000);
                    },
                    onClose: function () {
                        // 初始化
                        $(window).off('orientationchange.playvideosize');
                        self.initStyle();
                        clearLoadingTimer();
                    }
                });

                // 清除loading异常timer
                function clearLoadingTimer() {
                    if (self.loadingTimer) {
                        clearTimeout(self.loadingTimer);
                        self.loadingTimer = null;
                    }
                }
                // 添加视频节点
                function addVideo() {
                    popup.$popupContent.html(video);

                    // 关闭按钮距离上方的距离
                    var freesize = 0;
                    var remove = $('.c-videoplayer-popup .c-popup-remove');
                    if (remove.length) {
                        freesize = remove.height() + remove.offset().top;
                    }

                    // 屏幕旋转后，设置video的位置和尺寸
                    $(window).off('orientationchange.playvideosize resize.playvideosize')
                        .on('orientationchange.playvideosize resize.playvideosize', function () {
                            self.calculateVideo({
                                freesize: freesize
                            });
                        });

                    // 设置video的位置和尺寸
                    self.calculateVideo({
                        freesize: freesize
                    });
                }

                // 解决android手百关闭时页面抖动问题  待popup组件升级后更改 @士浩
                var remove = $('.c-videoplayer-popup .c-popup-remove');
                remove.length && remove.off('click.playvideo')
                    .on('click.playvideo', function () {
                        popup.$popupContent.length && popup.$popupContent.html('');
                    });
            });
        },

        initStyle: function () {
            var self = this;
            var video = self.$video;
            video.removeAttr('style');
        },

        // android全屏时计算video的高度和位置
        // @param {boolean}    params.freesize  可选 上下需要空闲出来的尺寸
        calculateVideo: function (params) {
            var self = this;
            var video = self.$video;
            var params = params || {};

            // 初始化样式避免被设定影响
            self.initStyle();

            setTimeout(function () {
                var winHeight = $(window).height();
                var winWidth = $(window).width();
                var freesize = params.freesize || 0;
                var h = video.height();

                if (winHeight <= winWidth) {
                    h = winHeight - freesize * 2;
                }
                video.css({
                    'height': h + 'px',
                    'margin-top': - h / 2 + 'px',
                    'visibility': 'visible'
                });

                // $('.c-popup-title').html(
                //     'vh:' + video.height() + 'vt:' + video.offset().top + 'f:' + freesize
                //     + 'wh:' + $(window).height() + 'ww:' + $(window).width());
            }, 10);
        },

        // 在手百中播放时 调用手百播放器 目前只限ios端
        // params.vid 视频唯一id     可选
        // params.src 视频资源地址   可选
        // params.title 视频标题     必填
        playInBox: function (params) {
            var url = params.src;
            if (!url) {
                return;
            }

            if (url.indexOf('//') === 0) {
                url = location.protocol + url;
            }

            require('pmd/deps/aio');

            var options = {
                vid: params.vid || '',
                isHalfScreen: false,
                title: '' || params.title,
                src: url,
                pageUrl: location.href,
                videoUrl: url,
                ubcMonitorInfo: {}
            };

            options = JSON.stringify(options);
            if (window.Box && window.Box.os && window.Box.os.ios) {
                Box.ios.invokeApp("video", {
                    "action": "invokePlayer",
                    "params": encodeURIComponent(options),
                    "minver": "8.0.0.6"
                });
            } 
        },

        pause: function() {
            this.video.pause();
        },
        remove: function() {
            this.$video.remove();
            this.isWorking = false;
        },
        // 获取视频时长
        getVideoTime: function() {
            return Math.ceil(this.video.duration);
        },
        on: function(name, fn) {
            this.$video.on(name, fn);
        },
        trigger: function(name) {
            this.$video.trigger(name);
        }
    }
    return VideoPlayer;
});